/*
 * SPDX-FileCopyrightText: 2021-2022 Espressif Systems (Shanghai) CO LTD
 *
 * SPDX-License-Identifier: Unlicense OR CC0-1.0
 */

#include <stdint.h>
#include <stdlib.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "driver/i2s_tdm.h"
#include "driver/gpio.h"
#include "driver/i2c.h"
#include "driver/gptimer.h"
#include "driver/uart.h"
#include "esp_check.h"
#include "sdkconfig.h"

#define EXAMPLE_BUFF_SIZE               2048

static i2s_chan_handle_t                tx_chan;        // I2S tx channel handler
static i2s_chan_handle_t                rx_chan;        // I2S rx channel handler


static void i2s_example_init_tdm_duplex(void)
{
//    /* Setp 1: Determine the I2S channel configuration and allocate both channels
//     * The default configuration can be generated by the helper macro,
//     * it only requires the I2S controller id and I2S role */
//    i2s_chan_config_t chan_cfg = I2S_CHANNEL_DEFAULT_CONFIG(I2S_NUM_AUTO, I2S_ROLE_MASTER);
//    ESP_ERROR_CHECK(i2s_new_channel(&chan_cfg, &tx_chan, &rx_chan));
//
//    /* Step 2: Setting the configurations of TDM mode, and initialize rx & tx channels
//     * The slot configuration and clock configuration can be generated by the macros
//     * These two helper macros is defined in 'i2s_tdm.h' which can only be used in TDM mode.
//     * They can help to specify the slot and clock configurations for initialization or re-configuring */
//    i2s_tdm_config_t tdm_cfg = {
//        .clk_cfg  = I2S_TDM_CLK_DEFAULT_CONFIG(48000),
//        /* For the target that not support full data bit-width in multiple slots (e.g. ESP32C3, ESP32S3, ESP32C6)
//         * The maximum bits in one frame is limited by the hardware, the number of bit clock can't exceed 128 in one frame,
//         * which is to say, TDM mode can only support 32 bit-width data upto 4 slots,
//         * 16 bit-width data upto 8 slots and 8 bit-width data upto 16 slots
//         * But for the target that support full data bit-width in multiple slots (e.g. ESP32H2)
//         * There is no such limitation, it can support up to 32 bit-width with 16 slots */
//        .slot_cfg = I2S_TDM_MSB_SLOT_DEFAULT_CONFIG(I2S_DATA_BIT_WIDTH_16BIT, I2S_SLOT_MODE_STEREO,
//                                                    I2S_TDM_SLOT0 | I2S_TDM_SLOT1 | I2S_TDM_SLOT2 | I2S_TDM_SLOT3 | I2S_TDM_SLOT4),
//        .gpio_cfg = {
//            .mclk = I2S_GPIO_UNUSED,    // some codecs may require mclk signal, this example doesn't need it
//            .bclk = GPIO_NUM_6,
//            .ws   = GPIO_NUM_9,
//            .dout = GPIO_NUM_8,
//            .din  = GPIO_NUM_7, // In duplex mode, bind output and input to a same gpio can loopback internally
//            .invert_flags = {
//                .mclk_inv = false,
//                .bclk_inv = false,
//                .ws_inv   = false,
//            },
//        },
//    };
//    /* While there are more than two slots activated, receiving data might be wrong if the mclk multiple is too small
//     * The driver will increasing the multiple automatically to ensure the bclk_div is greater than 2.
//     * Modify the mclk_multiple to 512 directly here to avoid the warning */
//    tdm_cfg.clk_cfg.mclk_multiple = I2S_MCLK_MULTIPLE_512;


    i2s_chan_config_t chan_cfg = {
        .id=I2S_NUM_AUTO,                 /*!< I2S port id */
        .role = I2S_ROLE_MASTER,               /*!< I2S role, I2S_ROLE_MASTER or I2S_ROLE_SLAVE */

        /* DMA configurations */
        .dma_desc_num = 6,       /*!< I2S DMA buffer number, it is also the number of DMA descriptor */
        .dma_frame_num = 240,      /*!< I2S frame number in one DMA buffer. One frame means one-time sample data in all slots,
                                                 *   it should be the multiple of '3' when the data bit width is 24.
                                                 */
        .auto_clear = true,         /*!< Set to auto clear DMA TX buffer, i2s will always send zero automatically if no data to send */
    };

    ESP_ERROR_CHECK(i2s_new_channel(&chan_cfg, &tx_chan, &rx_chan));

    i2s_tdm_config_t tdm_cfg = {
            .clk_cfg  = {
                    .sample_rate_hz = 48000,     /*!< I2S sample rate */
                    .clk_src = I2S_CLK_SRC_DEFAULT,            /*!< Choose clock source */
                    .mclk_multiple = I2S_MCLK_MULTIPLE_384, //**Ignored in slave mode
            },
            .slot_cfg = {
                    .data_bit_width = 16,     /*!< I2S sample data bit width (valid data bits per sample) */
                    .slot_bit_width = 16,     /*!< I2S slot bit width (total bits per slot) */
                    .slot_mode = I2S_SLOT_MODE_STEREO,          /*!< Set mono or stereo mode with I2S_SLOT_MODE_MONO or I2S_SLOT_MODE_STEREO */
                    .slot_mask = I2S_TDM_SLOT0|I2S_TDM_SLOT1|I2S_TDM_SLOT2|I2S_TDM_SLOT3|I2S_TDM_SLOT4, /*!< Slot mask. Activating slots by setting 1 to corresponding bits. When the activated slots is not consecutive, those data in unactivated slots will be ignored */
                    .ws_width = 1,           /*!< WS signal width (i.e. the number of bclk ticks that ws signal is high) */
                    .ws_pol = true,             /*!< WS signal polarity, set true to enable high lever first */
                    .bit_shift = true,          /*!< Set true to enable bit shift in Philips mode */
                    .left_align = true,         /*!< Set true to enable left alignment */
                    .big_endian = false,         /*!< Set true to enable big endian */
                    .bit_order_lsb = false,      /*!< Set true to enable lsb first */
                    .skip_mask = true,          /*!< Set true to enable skip mask. If it is enabled, only the data of the enabled channels will be sent, otherwise all data stored in DMA TX buffer will be sent */
                    .total_slot = 8,         /*!< I2S total number of slots. If it is smaller than the biggest activated channel number, it will be set to this number automatically. */
            },
            .gpio_cfg = {
                    .mclk = I2S_GPIO_UNUSED,
                    .bclk = GPIO_NUM_6,
                    .ws   = GPIO_NUM_9,
                    .dout = GPIO_NUM_8,
                    .din  = GPIO_NUM_7,
                    .invert_flags = {
                            .mclk_inv = false,
                            .bclk_inv = false,
                            .ws_inv   = false,
                    },
            },
    };

    /* Initialize the channels */
    ESP_ERROR_CHECK(i2s_channel_init_tdm_mode(tx_chan, &tdm_cfg));
    ESP_ERROR_CHECK(i2s_channel_init_tdm_mode(rx_chan, &tdm_cfg));
}


void app_main(void)
{
    i2s_example_init_tdm_duplex();

    /* Enable the TX channel */
    ESP_ERROR_CHECK(i2s_channel_enable(tx_chan));
    /* Enable the RX channel */
    ESP_ERROR_CHECK(i2s_channel_enable(rx_chan));

    while(1);

    /* Step 3: Create writing and reading task, enable and start the channels */
  //  xTaskCreate(i2s_example_read_task, "i2s_example_read_task", 4096, NULL, 5, NULL);
  //  xTaskCreate(i2s_example_write_task, "i2s_example_write_task", 4096, NULL, 5, NULL);
}
